#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add
from gvsbuild.utils.utils import file_replace


@project_add
class Libarchive(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "libarchive",
            version="3.8.5",
            repository="https://github.com/libarchive/libarchive",
            archive_url="https://libarchive.org/downloads/libarchive-{version}.tar.xz",
            hash="d68068e74beee3a0ec0dd04aee9037d5757fcc651591a6dcf1b6d542fb15a703",
            dependencies=[
                "cmake",
                "ninja",
                "win-iconv",
                "zlib",
                "lz4",
                "openssl",
                "libxml2",
            ],
            patches=[],
        )

    def build(self):
        cmake_params = (
            "-DENABLE_WERROR=OFF -DENABLE_TEST=OFF -DBUILD_TESTING=OFF -Wno-dev"
        )
        CmakeProject.build(self, cmake_params=cmake_params, use_ninja=True)
        # Fix the pkg-config .pc file, correcting the library's names
        file_replace(
            os.path.join(self.pkg_dir, "lib", "pkgconfig", "libarchive.pc"),
            [
                (" -llz4", " -lliblz4"),
                (" -leay32", " -llibeay32"),
                (" -lxml2", " -llibxml2"),
            ],
        )
        self.install(r".\COPYING share\doc\libarchive")



#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.


from gvsbuild.utils.base_group import Group, group_add
from gvsbuild.utils.base_project import Project, ProjectType


def test_group_creation():
    """Test Group creation."""
    group = Group("test-group")
    assert group.name == "test-group"


def test_group_unpack():
    """Test Group unpack does nothing."""
    group = Group("test-group")
    # Should not raise error and return None
    result = group.unpack()
    assert result is None


def test_group_build():
    """Test Group build does nothing."""
    group = Group("test-group")
    # Should not raise error and return None
    result = group.build()
    assert result is None


def test_group_export():
    """Test Group export does nothing."""
    group = Group("test-group")
    # Should not raise error and return None
    result = group.export()
    assert result is None


def test_group_add_decorator():
    """Test group_add decorator registers group."""

    # Create a test group with unique name
    @group_add
    class TestGroupDecorator(Group):
        def __init__(self):
            Group.__init__(self, "test-group-decorator-unique-name")

    # The class should be registered
    assert TestGroupDecorator is not None


def test_group_with_dependencies():
    """Test Group with dependencies."""
    group = Group("test-group-with-deps", dependencies=["dep1", "dep2"])
    assert group.dependencies == ["dep1", "dep2"]


def test_real_groups_exist():
    """Test that real groups from the project exist."""
    Project.add_all()
    groups = [p for p in Project._projects if p.type == ProjectType.GROUP]

    # Should have at least some groups
    assert len(groups) > 0

    # Check group names
    group_names = [g.name for g in groups]
    # Common groups in gvsbuild
    # Note: actual group names may vary, this test just ensures groups exist
    assert len(group_names) > 0



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import io
import sys

import pytest


class CycloptsRunner:
    """Test runner for Cyclopts apps."""

    class Result:
        def __init__(self, exit_code, stdout, stderr):
            self.exit_code = exit_code
            self.output = stdout
            self.stdout = stdout
            self.stderr = stderr

    def invoke(self, app, args, console=None, **kwargs):
        """Invoke a Cyclopts app with given arguments.

        Args:
            app: The Cyclopts App instance to invoke
            args: List of command-line arguments
            console: Optional Rich Console for consistent output formatting
            **kwargs: Additional keyword arguments (ignored for compatibility)

        Returns:
            Result object with exit_code, output, stdout, and stderr
        """
        stdout = io.StringIO()
        stderr = io.StringIO()
        exit_code = 0

        old_stdout = sys.stdout
        old_stderr = sys.stderr

        try:
            sys.stdout = stdout
            sys.stderr = stderr

            # Call app with args list (not empty call to avoid warning)
            # Pass console if provided for consistent Rich formatting
            if console:
                app(args if args else [], console=console)
            else:
                app(args if args else [])

        except SystemExit as e:
            exit_code = e.code if e.code is not None else 0
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr

        return self.Result(exit_code, stdout.getvalue(), stderr.getvalue())


@pytest.fixture
def app():
    """Fixture providing the gvsbuild Cyclopts app instance."""
    from gvsbuild.main import app as gvsbuild_app

    yield gvsbuild_app


@pytest.fixture(scope="session")
def tmp_dir(tmp_path_factory):
    """Fixture providing a temporary directory for testing."""
    return tmp_path_factory.mktemp("gtk-build")


@pytest.fixture
def runner():
    """Fixture providing a CycloptsRunner for testing."""
    return CycloptsRunner()


@pytest.fixture
def console():
    """Fixture providing a Rich console with consistent settings for testing."""
    from rich.console import Console

    return Console(
        width=80,
        force_terminal=True,
        highlight=False,
        color_system=None,
        legacy_windows=False,
    )


@pytest.fixture
def mock_opts(mocker, tmp_path):
    """Fixture providing a mock opts object with common attributes."""
    opts = mocker.Mock()
    opts.tools_root_dir = str(tmp_path / "tools")
    opts.build_dir = str(tmp_path / "build")
    opts.msys_dir = str(tmp_path / "msys64")
    opts.gtk_dir = str(tmp_path / "gtk")
    opts.archives_download_dir = str(tmp_path / "downloads")
    opts.patches_root_dir = str(tmp_path / "patches")
    return opts


@pytest.fixture
def mock_project(mocker):
    """Fixture providing a mock project with common attributes."""
    project = mocker.Mock()
    project.dependencies = []
    project.all_dependencies = []
    project.name = "test-project"
    return project



#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import CmakeProject
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class Nuspell(Tarball, CmakeProject):
    def __init__(self):
        Project.__init__(
            self,
            "nuspell",
            version="5.1.7",
            repository="https://github.com/nuspell/nuspell",
            archive_url="https://github.com/nuspell/nuspell/archive/v{version}.tar.gz",
            archive_filename="nuspell-{version}.tar.gz",
            hash="9aee944e86924ce44434741cb950fee8f9a6ff9c4f002803ab5f04698c8e4c68",
            dependencies=["cmake", "ninja", "icu"],
        )

    def build(self):
        cmake_params = (
            f"-DCMAKE_INSTALL_PREFIX={self.builder.gtk_dir} "
            "-DBUILD_SHARED_LIBS=ON -DBUILD_TESTING=OFF -DBUILD_DOCS=OFF "
            f"-DBUILD_TOOLS=OFF -DICU_ROOT={self.builder.gtk_dir}"
        )
        CmakeProject.build(self, use_ninja=True, cmake_params=cmake_params)

        self.install(r".\COPYING*", r".\share\doc\nuspell")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

import os

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class GStreamer(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gstreamer",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gstreamer/gstreamer-{version}.tar.xz",
            hash="6c8676bc39a2b41084fd4b21d2c37985c69ac979c03ce59575db945a3a623afd",
            dependencies=["meson", "ninja", "glib", "orc"],
            patches=[],
        )

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        enable_gi = "enabled" if self.opts.enable_gi else "disabled"
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        add_path = os.path.join(self.builder.opts.msys_dir, "usr", "bin")

        Meson.build(
            self, add_path=add_path, meson_params="-Dtests=disabled -Dexamples=disabled"
        )
        self.install(r".\COPYING share\doc\gstreamer")


@project_add
class Orc(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "orc",
            version="0.4.42",
            lastversion_even=True,
            repository="https://gitlab.freedesktop.org/gstreamer/orc",
            archive_url="https://gstreamer.freedesktop.org/src/orc/orc-{version}.tar.xz",
            hash="7ec912ab59af3cc97874c456a56a8ae1eec520c385ec447e8a102b2bd122c90c",
            dependencies=["meson", "ninja"],
        )

    def build(self):
        Meson.build(self, meson_params="-Dbenchmarks=disabled -Dtools=enabled")
        self.install(r"COPYING share\doc\orc")


@project_add
class GstPluginsBase(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-base",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-base/gst-plugins-base-{version}.tar.xz",
            hash="eace79d63bd2edeb2048777ea9f432d8b6e7336e656cbc20da450f6235758b31",
            dependencies=[
                "meson",
                "ninja",
                "gstreamer",
                "opus",
                "ogg",
                "graphene",
                "pango",
            ],
        )
        # Examples depend on GTK3
        self.add_param("-Dexamples=disabled")

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        enable_gi = "enabled" if self.opts.enable_gi else "disabled"
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(
            self, meson_params=f'-Dc_link_args="{self.builder.gtk_dir}\\lib\\ogg.lib"'
        )
        self.install(r".\COPYING share\doc\gst-plugins-base")


@project_add
class GstPluginsGood(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-good",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-good/gst-plugins-good-{version}.tar.xz",
            hash="d97700f346fdf9ef5461c035e23ed1ce916ca7a31d6ddad987f774774361db77",
            dependencies=[
                "meson",
                "ninja",
                "gst-plugins-base",
                "libvpx",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-good")


@project_add
class GstPluginsBad(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-bad",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-bad/gst-plugins-bad-{version}.tar.xz",
            hash="32d825041e5775fc9bf9e8c38e3a5c46c1441eee67f8112572450a9c23c835f0",
            dependencies=[
                "meson",
                "ninja",
                "gst-plugins-base",
                "libnice",
                "webrtc-audio-processing",
                "openssl",
                "libsrtp2",
            ],
            patches=[
                "wasapisink-reduce-buffer-latency.patch",
            ],
        )
        self.add_param("-Dcurl=disabled")
        self.add_param("-Dcurl-ssh2=disabled")

        # Adding webrtc-audio-processing adds the isac plugin. It fails to compile.
        self.add_param("-Disac=disabled")

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        enable_gi = "enabled" if self.opts.enable_gi else "disabled"
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-bad")


@project_add
class GstPluginsUgly(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugins-ugly",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-plugins-ugly/gst-plugins-ugly-{version}.tar.xz",
            hash="743f28b93c941e0af385ab193a2150f9f79bc6269adc639f6475d984794c217c",
            dependencies=["meson", "ninja", "gst-plugins-base"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-plugins-ugly")


@project_add
class GstDevTools(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-devtools",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-devtools/gst-devtools-{version}.tar.xz",
            hash="184a68b2c967210c6ddacad9a8e2c7ebc5e8df9b10fd7b72b7f7580cc0a60fb1",
            dependencies=[
                "meson",
                "ninja",
                "json-glib",
                "gst-plugins-base",
                "gst-plugins-bad",
                "gst-rtsp-server",
            ],
        )

        if self.opts.enable_gi:
            self.add_dependency("gobject-introspection")
        enable_gi = "enabled" if self.opts.enable_gi else "disabled"
        self.add_param(f"-Dintrospection={enable_gi}")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-devtools")


@project_add
class GstPython(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-python",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-python/gst-python-{version}.tar.xz",
            hash="9eba882a413cf06bf0575e635f73c0a2d01f3abdf76e18f804a90f3ff6a0aa2d",
            dependencies=[
                "meson",
                "ninja",
                "pygobject",
                "gst-plugins-base",
                "gst-plugins-bad",
                "gst-rtsp-server",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-python")


@project_add
class GstLibav(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-libav",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-libav/gst-libav-{version}.tar.xz",
            hash="e3c93db7da2da3b2374ccc2e7394316f9192460abdea81651652791d46ccb8fb",
            # TODO try remove gst-plugins-base
            dependencies=["meson", "ninja", "pygobject", "ffmpeg", "gst-plugins-base"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-libav")


@project_add
class GstRtspServer(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-rtsp-server",
            repository="https://gitlab.freedesktop.org/gstreamer/gstreamer",
            version="1.28.0",
            lastversion_even=True,
            archive_url="https://gstreamer.freedesktop.org/src/gst-rtsp-server/gst-rtsp-server-{version}.tar.xz",
            hash="5ed0938ea0fc1df2709cc939245d93294f09b2d7220e19f7fcdb306ce2b6cee5",
            dependencies=["meson", "ninja", "gstreamer"],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\gst-rtsp-server")


@project_add
class GstPluginGtk4(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "gst-plugin-gtk4",
            version="0.14.3",
            repository="https://gitlab.freedesktop.org/gstreamer/gst-plugins-rs",
            archive_url="https://gitlab.freedesktop.org/gstreamer/gst-plugins-rs/-/archive/{version}/gst-plugins-rs-{version}.tar.gz?ref_type=tags",
            archive_filename="gst-plugins-rs-{version}.tar.gz",
            hash="0609b2459ec29f4678edd98e69b6b0a473ef4a303d60645260245dbc23f75167",
            dependencies=[
                "meson",
                "cargo",
                "gst-plugins-base",
                "gst-plugins-bad",
                "gtk4",
            ],
        )
        self.add_param("-Dgtk4=enabled")
        self.add_param("--auto-features=disabled")

    def build(self):
        self.builder.exec_cargo("install cargo-c --locked")
        Meson.build(self)
        self.install(r".\LICENSE-MPL-2.0 share\doc\gst-plugin-gtk4")



#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
import os

import pytest

from gvsbuild.utils.utils import (
    convert_to_msys,
    file_replace,
    ordered_set,
    python_find_libs_dir,
    read_file,
    rmtree_full,
    write_file,
)


def test_convert_to_msys():
    """Test converting Windows paths to MSYS format."""
    result = convert_to_msys("C:\\Users\\test")
    assert result == "/C/Users/test"

    result = convert_to_msys("D:\\Program Files")
    assert result == "/D/Program Files"


def test_convert_to_msys_invalid():
    """Test converting invalid path raises error."""
    with pytest.raises(NotADirectoryError):
        convert_to_msys("/invalid/path")


def test_read_write_file(tmp_path):
    """Test reading and writing files."""
    test_file = tmp_path / "test.txt"
    content = ["line1", "line2", "line3"]

    # Write file
    write_file(test_file, content)
    assert test_file.exists()

    # Read file
    result = read_file(test_file)
    assert result == content


def test_file_replace(tmp_path):
    """Test file replacement functionality."""
    test_file = tmp_path / "replace.txt"
    original_content = ["Hello World", "Python is great", "Hello Python"]
    write_file(test_file, original_content)

    # Replace "Hello" with "Hi"
    file_replace(test_file, [("Hello", "Hi")])

    result = read_file(test_file)
    assert result == ["Hi World", "Python is great", "Hi Python"]

    # Backup file should exist
    backup_file = tmp_path / "replace.txt.bak"
    assert backup_file.exists()


def test_file_replace_no_backup(tmp_path):
    """Test file replacement without backup."""
    test_file = tmp_path / "replace2.txt"
    original_content = ["foo bar"]
    write_file(test_file, original_content)

    file_replace(test_file, [("foo", "baz")], make_bak=False)

    result = read_file(test_file)
    assert result == ["baz bar"]

    # Backup file should not exist
    backup_file = tmp_path / "replace2.txt.bak"
    assert not backup_file.exists()


def test_file_replace_no_changes(tmp_path):
    """Test file replacement with no matches."""
    test_file = tmp_path / "replace3.txt"
    original_content = ["foo bar"]
    write_file(test_file, original_content)

    file_replace(test_file, [("notfound", "replacement")])

    result = read_file(test_file)
    assert result == original_content


def test_file_replace_regex(tmp_path):
    """Test file replacement with regex."""
    test_file = tmp_path / "replace4.txt"
    original_content = ["value=123", "value=456"]
    write_file(test_file, original_content)

    file_replace(test_file, [(r"value=\d+", "value=999")])

    result = read_file(test_file)
    assert result == ["value=999", "value=999"]


def test_ordered_set():
    """Test ordered_set functionality."""
    os = ordered_set()

    # Add items
    os.add("first")
    os.add("second")
    os.add("third")

    # Check order is preserved
    items = list(os)
    assert items == ["first", "second", "third"]

    # Adding duplicate should not change order
    os.add("first")
    items = list(os)
    assert items == ["first", "second", "third"]


def test_ordered_set_remove():
    """Test ordered_set remove functionality."""
    os = ordered_set()
    os.add("a")
    os.add("b")
    os.add("c")

    os.remove("b")
    items = list(os)
    assert items == ["a", "c"]


def test_ordered_set_contains():
    """Test ordered_set membership test."""
    os = ordered_set()
    os.add("test")

    assert "test" in os
    assert "nothere" not in os


def test_rmtree_full(tmp_path):
    """Test rmtree_full functionality."""
    # Create directory with file
    test_dir = tmp_path / "test_rmtree"
    test_dir.mkdir()
    (test_dir / "file.txt").write_text("content")

    # Remove directory
    rmtree_full(str(test_dir))

    # Directory should not exist
    assert not test_dir.exists()


def test_rmtree_full_nonexistent(tmp_path):
    """Test rmtree_full with non-existent directory."""
    nonexistent = tmp_path / "doesnotexist"

    # Should not raise error
    rmtree_full(str(nonexistent))


def test_rmtree_full_readonly(tmp_path):
    """Test rmtree_full with read-only files."""
    test_dir = tmp_path / "test_readonly"
    test_dir.mkdir()
    test_file = test_dir / "readonly.txt"
    test_file.write_text("content")

    # Make file read-only
    os.chmod(test_file, 0o444)

    # Should still be able to remove
    rmtree_full(str(test_dir))
    assert not test_dir.exists()


def test_rmtree_full_with_retry(tmp_path):
    """Test rmtree_full with retry option."""
    test_dir = tmp_path / "test_retry"
    test_dir.mkdir()
    (test_dir / "file.txt").write_text("content")

    rmtree_full(str(test_dir), retry=True)
    assert not test_dir.exists()


def test_python_find_libs_dir_direct(tmp_path):
    """Test finding Python libs directory directly."""
    # Create fake Python directory with libs
    python_dir = tmp_path / "python"
    python_dir.mkdir()
    libs_dir = python_dir / "libs"
    libs_dir.mkdir()

    result = python_find_libs_dir(str(python_dir))
    assert result == str(libs_dir)


def test_python_find_libs_dir_virtualenv(tmp_path):
    """Test finding Python libs directory from virtualenv."""
    # Create fake virtualenv
    venv_dir = tmp_path / "venv"
    venv_dir.mkdir()

    # Create fake Python directory with libs
    python_dir = tmp_path / "python"
    python_dir.mkdir()
    libs_dir = python_dir / "libs"
    libs_dir.mkdir()

    # Create pyvenv.cfg pointing to python_dir
    pyvenv_cfg = venv_dir / "pyvenv.cfg"
    pyvenv_cfg.write_text(f"home = {python_dir}\n")

    result = python_find_libs_dir(str(venv_dir))
    assert result == str(libs_dir)


def test_python_find_libs_dir_not_found(tmp_path):
    """Test python_find_libs_dir when libs directory doesn't exist."""
    python_dir = tmp_path / "python"
    python_dir.mkdir()

    result = python_find_libs_dir(str(python_dir))
    # Should return path even if it doesn't exist, or None
    assert result is None or not os.path.isdir(result)



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.


from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import project_add


@project_add
class AdwaitaIconTheme(Tarball, Meson):
    def __init__(self):
        Meson.__init__(
            self,
            "adwaita-icon-theme",
            version="49.0",
            repository="https://gitlab.gnome.org/GNOME/adwaita-icon-theme",
            archive_url="https://download.gnome.org/sources/adwaita-icon-theme/{major}/adwaita-icon-theme-{version}.tar.xz",
            hash="65166461d1b278aa942f59aa8d0fccf1108d71c65f372c6266e172449791755c",
            dependencies=[
                "hicolor-icon-theme",
                "librsvg",
                "gtk4",
            ],
            patches=[
                # https://gitlab.gnome.org/GNOME/adwaita-icon-theme/-/merge_requests/89
                "001-fix-svg-symlink.patch",
            ],
        )

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING_CCBYSA3 share\doc\adwaita-icon-theme")



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.

from gvsbuild.utils.base_builders import Meson
from gvsbuild.utils.base_expanders import Tarball
from gvsbuild.utils.base_project import Project, project_add


@project_add
class PkgConf(Tarball, Meson):
    def __init__(self):
        Project.__init__(
            self,
            "pkgconf",
            prj_dir="pkgconf",
            version="2.5.1",
            repository="https://github.com/pkgconf/pkgconf",
            archive_url="https://distfiles.ariadne.space/pkgconf/pkgconf-{version}.tar.gz",
            hash="ab89d59810d9cad5dfcd508f25efab8ea0b1c8e7bad91c2b6351f13e6a5940d8",
            dependencies=["ninja", "meson"],
            patches=["0001-libpkgconf-add-defines-to-unbreak-build-with-VS2013.patch"],
        )
        self.add_param("-Dtests=disabled")

    def build(self):
        Meson.build(self)
        self.install(r".\COPYING share\doc\pkgconf")

    def post_install(self):
        self.exec_cmd(
            r"copy %(gtk_dir)s\bin\pkgconf.exe %(gtk_dir)s\bin\pkg-config.exe"
        )



#  Copyright (C) 2025 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.
import os

import pytest

from gvsbuild.utils.base_tool import Tool, tool_add


def test_tool_creation():
    tool = Tool("test-tool")
    assert tool.name == "test-tool"
    assert tool.mark_deps is False


def test_tool_with_dir_part():
    tool = Tool("test-tool", version="1.0.0", dir_part="test-{version}")
    assert tool.dir_part == "test-1.0.0"


def test_tool_load_defaults_with_dir_part(mock_opts, tmp_path):
    tool = Tool("test-tool", version="1.0.0", dir_part="test-{version}")
    mock_opts.tools_root_dir = str(tmp_path)
    tool.opts = mock_opts

    tool.load_defaults()
    assert str(tmp_path / "test-1.0.0") in tool.build_dir


def test_tool_load_defaults_without_dir_part(mock_opts, tmp_path):
    tool = Tool("test-tool")
    mock_opts.tools_root_dir = str(tmp_path)
    tool.opts = mock_opts

    tool.load_defaults()
    assert str(tmp_path / "test-tool") in tool.build_dir


def test_tool_with_exe_name(mock_opts, tmp_path):
    tool = Tool("test-tool", exe_name="test.exe")
    mock_opts.tools_root_dir = str(tmp_path)
    tool.opts = mock_opts

    tool.load_defaults()
    assert tool.full_exe is not None
    assert "test.exe" in tool.full_exe


def test_tool_mark(tmp_path):
    tool = Tool("test-tool")
    tool.build_dir = str(tmp_path / "test-tool-mark")

    assert not os.path.exists(tool.build_dir)
    tool.tool_mark()
    assert os.path.exists(tool.build_dir)
    assert tool.mark_deps is True


def test_tool_mark_existing_dir(tmp_path):
    build_dir = tmp_path / "existing"
    build_dir.mkdir()

    tool = Tool("test-tool")
    tool.build_dir = str(build_dir)
    tool.mark_deps = False

    tool.tool_mark()
    assert tool.mark_deps is False


def test_tool_build_no_mark():
    tool = Tool("test-tool")
    tool.mark_deps = False

    result = tool.build()
    assert result is True


def test_tool_build_with_mark():
    tool = Tool("test-tool")
    tool.mark_deps = True

    result = tool.build()
    assert result is False


def test_tool_get_path_with_tool_path():
    tool = Tool("test-tool")
    tool.tool_path = "/path/to/tool"
    tool.build_dir = "/build/dir"

    result = tool.get_path()
    assert result == "/path/to/tool"


def test_tool_get_path_without_tool_path():
    tool = Tool("test-tool")
    tool.tool_path = None
    tool.build_dir = "/build/dir"

    result = tool.get_path()
    assert result == "/build/dir"


def test_tool_get_executable():
    tool = Tool("test-tool", exe_name="test.exe")
    tool.full_exe = "/path/to/test.exe"

    result = tool.get_executable()
    assert result == "/path/to/test.exe"


def test_tool_get_executable_no_exe():
    tool = Tool("test-tool")
    tool.full_exe = None

    with pytest.raises(NotImplementedError):
        tool.get_executable()


def test_tool_get_base_dir_not_implemented():
    tool = Tool("test-tool")

    with pytest.raises(NotImplementedError):
        tool.get_base_dir()


def test_tool_export():
    tool = Tool("test-tool")
    result = tool.export()
    assert result is None


def test_tool_update_build_dir(mocker):
    tool = Tool("test-tool")
    mock_unpack = mocker.patch.object(tool, "unpack")

    tool.update_build_dir()

    mock_unpack.assert_called_once()


def test_tool_add_decorator():
    @tool_add
    class TestToolDecorator(Tool):
        def __init__(self):
            Tool.__init__(self, "test-tool-decorator-unique")

        def unpack(self):
            pass

    assert TestToolDecorator is not None



#  Copyright (C) 2016 The Gvsbuild Authors
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, see <http://www.gnu.org/licenses/>.


def test_build_help(app, runner, console):
    """Test build command help output with consistent console formatting."""
    result = runner.invoke(app, ["build", "--help"], console=console)
    assert result.exit_code == 0
    assert "Build a project or a list of projects" in result.output
    assert "PROJECTS" in result.output
    assert "--platform" in result.output


def test_wrong_project_name(app, runner):
    result = runner.invoke(app, ["build", "bad-name"], color=True)
    assert result.exit_code == 1
    full_output = result.output + result.stderr
    assert "not a valid project name" in full_output


def test_no_project(app, runner):
    result = runner.invoke(app, ["build"])
    # Cyclopts returns exit code 1 for errors
    assert result.exit_code in [1, 2]
    full_output = result.output + result.stderr
    assert (
        "requires an argument" in full_output.lower()
        or "required" in full_output.lower()
        or "missing" in full_output.lower()
        or "at least one project" in full_output.lower()
    )


def test_platform(tmp_dir, app, runner):
    # This test just ensures the command can be invoked with platform argument
    # Actual building may fail due to missing tools/dependencies in test environment
    assert tmp_dir.exists()
    result = runner.invoke(
        app,
        [
            "build",
            "--build-dir",
            str(tmp_dir),
            "--platform",
            "x86",
            "hello-world",
        ],
    )
    # Exit code may be 0 (success) or 1 (build error), but should not be 2 (argument error)
    assert result.exit_code in [0, 1]


def test_ninja_opts_validation_valid_single_dash(app, runner):
    """Test that ninja-opts accepts valid single-dash options."""
    result = runner.invoke(app, ["build", "--ninja-opts", "-j2", "hello-world"])
    # Should not fail with validation error (exit code 1 is OK for build errors)
    assert result.exit_code != 2
    full_output = result.output + result.stderr
    assert "ninja-opts must start with a dash" not in full_output


def test_ninja_opts_validation_valid_double_dash(app, runner):
    """Test that ninja-opts accepts valid double-dash options."""
    result = runner.invoke(app, ["build", "--ninja-opts", "--verbose", "hello-world"])
    # Should not fail with validation error (exit code 1 is OK for build errors)
    assert result.exit_code != 2
    full_output = result.output + result.stderr
    assert "ninja-opts must start with a dash" not in full_output


def test_ninja_opts_validation_valid_equals_syntax(app, runner):
    """Test that ninja-opts accepts equals syntax."""
    result = runner.invoke(app, ["build", "--ninja-opts=-j2", "hello-world"])
    # Should not fail with validation error (exit code 1 is OK for build errors)
    assert result.exit_code != 2
    full_output = result.output + result.stderr
    assert "ninja-opts must start with a dash" not in full_output


def test_ninja_opts_validation_invalid_no_dash(app, runner):
    """Test that ninja-opts rejects values that don't start with dash."""
    result = runner.invoke(app, ["build", "--ninja-opts", "j2", "hello-world"])
    assert result.exit_code == 1  # Validation error
    full_output = result.output + result.stderr
    assert "ninja-opts must start with a dash (- or --)" in full_output
    assert "Got: 'j2'" in full_output
    assert "--ninja-opts -j2" in full_output
    assert "--ninja-opts --verbose" in full_output


def test_ninja_opts_validation_invalid_equals_syntax(app, runner):
    """Test that ninja-opts rejects invalid values with equals syntax."""
    result = runner.invoke(app, ["build", "--ninja-opts=j2", "hello-world"])
    assert result.exit_code == 1  # Validation error
    full_output = result.output + result.stderr
    assert "ninja-opts must start with a dash (- or --)" in full_output
    assert "Got: 'j2'" in full_output


def test_ninja_opts_validation_empty_value(app, runner):
    """Test that ninja-opts accepts empty/None values."""
    result = runner.invoke(app, ["build", "hello-world"])
    # Should not fail with validation error (exit code 1 is OK for build errors)
    assert result.exit_code != 2
    full_output = result.output + result.stderr
    assert "ninja-opts must start with a dash" not in full_output
